//******************************************************************************
// Compiler.java:	
//
//******************************************************************************
import java.awt.*;
import java.io.*;

/**
 * Oberon-00 Compiler for ICS280B
 * Assignments 1 and 2
 * by Joachim Feise
 * jfeise@ics.uci.edu
 * University of California, Irvine
 * Department of Information and Computer Science
 * Spring Quarter 1997
 * Instructor: Michael Franz
 */
class CompilerWindow extends Frame
{
	MenuItem menuitem1, menuitem2;
	MenuItem menuitem3;
    MenuItem menuitem4;

	// CompilerWindow constructor
	//--------------------------------------------------------------------------
	public CompilerWindow( String str )
	{
		super (str);
		setLayout( new BorderLayout() );
		setBackground( Color.lightGray );

		MenuBar menubar = new MenuBar();
		setMenuBar( menubar );
		Menu menu1 = new Menu( "File" );
		menubar.add( menu1 );
		menuitem1 = new MenuItem( "Compile" );
		menu1.add( menuitem1 );
//		menu1.addSeparator();
//		menuitem4 = new MenuItem( "Use Test file" );
//		menu1.add( menuitem4 );
		menu1.addSeparator();
		menuitem2 = new MenuItem( "Exit" );
		menu1.add( menuitem2 );

		Menu menu2 = new Menu( "Help" );
		menubar.setHelpMenu( menu2 );
		menuitem3 = new MenuItem( "About Oberon-00 Compiler" );
		menu2.add( menuitem3 );

	}


    public void RunCompiler()
    {
        RunCompiler( null, true );
    }

	public void RunCompiler( String name, boolean interactive )
	{
        String filename = null;
        String dirname = null;
        if( interactive )
        {
            FileDialog f = new FileDialog( this, "Select an input file", FileDialog.LOAD );
            f.setFile( "*.o00" );
            f.show();
            filename = f.getFile();
            if( filename == null || filename == "" )
                return;
            dirname = f.getDirectory();
            int slash = dirname.lastIndexOf( "\\" );
            if( slash == -1 )
                slash = dirname.lastIndexOf( "/" );
            if( slash >= 0 )
                dirname = dirname.substring( 0, slash + 1 );
            filename = dirname + filename;
        }
        else
            filename = name;

        int dot = filename.lastIndexOf( "." );
        String outname = filename.substring( 0, dot );
        outname += ".out";
        PrintStream ps = null;
		try
		{
			File outputFile = new File( outname );
			FileOutputStream fos = new FileOutputStream( outputFile );
            ps = new PrintStream( fos );
		}
		catch( IOException e )
		{
			System.err.println( "Output file creation: " + e );
		}

        ParseTree parsetree = new ParseTree( ps );
        parser p = new parser( filename, parsetree );
        try
        {
            p.parse();
        }
		catch( Exception e )
		{
            String s = "Parser error: " + e;
			System.err.println( s );
            ps.println( s );
		}

        String output = parsetree.TraverseNodes();
        ps.println( output );
        output = parsetree.TraverseBlocks();
        ps.println( output );
        output = parsetree.TraverseDominatorTree();
        ps.println( output );
        output = parsetree.TraverseUselist();
        ps.println( output );

        if( interactive )
			new Done( this, filename, outname );
    }

	public boolean handleEvent(Event evt)
	{
		switch (evt.id)
		{
			// Application shutdown (e.g. user chooses Close from the system menu).
			//------------------------------------------------------------------
			case Event.WINDOW_DESTROY:
				dispose();
				System.exit(0);
				return true;

			default:
				return super.handleEvent(evt);
		}			 
	}


	public boolean action( Event event, Object arg )
	{
		if( event.target instanceof MenuItem )
		{
			MenuItem menuitem = (MenuItem)(event.target);
			if( menuitem == menuitem1 )
			{
				RunCompiler();
				return true;
			}
			else if( menuitem == menuitem2 )
			{
				dispose();
				System.exit(0);
				return true;
			}
			else if( menuitem == menuitem3 )
			{
				new AboutDialog( this );
				return true;
			}
			if( menuitem == menuitem4 )
			{
				RunCompiler( "test.o00", false );
				return true;
			}
		}
		return false;
	}


	public static void main(String args[])
	{
		CompilerWindow window = new CompilerWindow( "Oberon-00 Compiler" );

        if( args.length >= 1 )
        {
            window.RunCompiler( args[0], false );
            System.exit(0);
        }
        else
        {
    		window.resize(300, 150 );
	    	window.show();
        }
	}
	public void paint( Graphics g )
	{
		update( g );
	}

	public void update( Graphics g )
	{
		Color fg = getForeground();
		Dimension d = size();
		Insets i = insets();
		g.setColor( Color.lightGray );
		g.fillRect( 0, 0, d.width-1, d.height-1 );

        String vendor = System.getProperty( "java.vendor", "dont know" );
        int left = 0;
        int top = 0;
        int width = d.width;
        int height = d.height;
        if( vendor.indexOf( "Sun" ) >= 0 )
        {
            // for Sun's java interpreter
            left = i.left;
            top = i.top;
		    width = d.width - left - i.right;
		    height = d.height - top - i.bottom;
        }
        else if( vendor.indexOf( "Microsoft" ) >= 0 )
        {
            // for Microsoft's jview interpreter
            left = 0;
            top = 0;
		    width = d.width - i.left - i.right;
		    height = d.height - i.top - i.bottom;
        }
        else
        {
            // for all other vendors supposed to work the same way as Sun
            left = i.left;
            top = i.top;
		    width = d.width - left - i.right;
		    height = d.height - top - i.bottom;
        }
        g.draw3DRect( left, top, width-1, height-1, true );
		g.draw3DRect( left+5, top+5, width-11, height-11, false );
		g.setColor( fg );
	}
}
