import NodeKind;
import Operator;
import Block;

/*
 *
 * Node
 *
 */
public class Node 
{
	protected Node m_prev;      // reference to previous and next nodes
	protected Node m_next;
	protected Block m_block;    // reference to parent block
	protected Node m_lastUse;   // reference to place where variable was last used
    protected boolean m_lValue; // is this node on the left side of an assignment?
    protected NodeKind m_kind;  // what kind of node is this?
    protected Operator m_op;    // specifies the operator for instruction nodes
	protected Node m_x;         // references to left and right subtrees/operands
	protected Node m_y;
    protected String m_value;   // node name/value
    protected boolean m_valueSet;
    protected int m_SSANumber;  // unique number for single static assignment
	protected Node m_OpLink;	// link to anchor table for CSE
	protected boolean m_erased;	// erased flag for CSE
    protected int m_number;     // node count
    private boolean m_visiting; // are we traversing this node?

	public static Node startNode = null;    // static start and endpoints of node list
	public static Node endNode = null;

    protected static int m_count = 0;       // current node counter


	public Node()
	{
		m_prev = m_next = null;
		m_block = null;
		m_lastUse = null;
        m_lValue = false;
		m_kind = null;
		m_op = null;
		m_x = m_y = null;
		m_value = Integer.toString(0);
        m_valueSet = false;
        m_SSANumber = 0;
		m_OpLink = null;
		m_erased = false;
        m_number = m_count++;
        m_visiting = false;
	}

    // update the block right away
    public Node( Block b )
	{
		m_prev = m_next = null;
		m_block = b;
        if( m_block != null )
            m_block.EndBlock( this );
		m_lastUse = null;
        m_lValue = false;
		m_kind = null;
		m_op = null;
		m_x = m_y = null;
		m_value = Integer.toString(0);
        m_SSANumber = 0;
		m_OpLink = null;
		m_erased = false;
        m_number = m_count++;
        m_visiting = false;
	}

    // get/set functions
    public NodeKind GetKind()
    {
        return m_kind;
    }

    public void SetKind( NodeKind kind )
    {
        m_kind = kind;
    }

    public Block GetBlock()
    {
        return m_block;
    }

    public void SetBlock( Block block )
    {
        m_block = block;
    }

    public Node GetLastUse()
    {
        return m_lastUse;
    }

    public void SetLastUse( Node node )
    {
        m_lastUse = node;
    }

    public boolean GetlValue()
    {
        return m_lValue;
    }

    public void SetlValue()
    {
        m_lValue = true;
    }

    public Node GetX()
    {
        return m_x;
    }

    public void SetX( Node x )
    {
        m_x = x;
    }

    public Node GetY()
    {
        return m_y;
    }

    public void SetY( Node y )
    {
        m_y = y;
    }

    public Operator GetOp()
    {
        return m_op;
    }

    public void SetOp( Operator op )
    {
        m_op = op;
    }

    public String GetValue()
    {
        return m_value;
    }

    public void SetValue( String value )
    {
        m_value = value;
        m_valueSet = true;
    }

    public boolean IsValueSet()
    {
        return m_valueSet;
    }

    public int GetSSANumber()
    {
        return m_SSANumber;
    }

    public void SetSSANumber( int number )
    {
        m_SSANumber = number;
    }

	public boolean IsErased()
	{
		return m_erased;
	}

	public void SetErased()
	{
		m_erased = true;
	}

	public Node GetOpLink()
	{
		return m_OpLink;
	}

	public void SetOpLink( Node opLink )
	{
		m_OpLink = opLink;
	}

    public Node GetNext()
    {
        return m_next;
    }

    public Node GetPrev()
    {
        return m_prev;
    }

    public void Visited()
    {
        m_visiting = true;
    }

    public boolean IsVisited()
    {
        return m_visiting;
    }

    // add a node to the end of the node list
    public void AddTail()
	{
		if( endNode != null )
		{
            endNode.m_next = this;
			m_prev = endNode;
			endNode = this;
		}
		else
		{
			startNode = endNode = this;
		}

        // add other additional nodes as well
        Node node = m_next;
        while( node != null )
        {
            endNode = node;
            node = node.m_next;
        }
	}

    public void InsertAfter( Node node, boolean updateList )
    {
        m_prev = node;
        m_next = node.m_next;
        node.m_next = this;
        if( m_next != null )
            m_next.m_prev = this;
        else
            if( updateList )
                endNode = this;
        if( m_block != null )
            if( m_prev == m_block.GetLast() )
                m_block.SetLast( this );
    }

    public void InsertBefore( Node node, boolean updateList )
    {
        m_prev = node.m_prev;
        m_next = node;
        if( m_prev != null )
            m_prev.m_next = this;
        else
            if( updateList )
                startNode = this;
        node.m_prev = this;
        if( m_block != null )
            if( m_next == m_block.GetFirst() )
                m_block.SetFirst( this );
    }

    // walk the node list
	public String Traverse()
	{
		String str = "Node List\n";
        str +=       "=========\n";
		Node node = startNode;
		while( node != null )
		{
			str += node.toString( 2 );
			node = node.m_next;
		}
		return str;
	}

    // return node information
	public String toString( int ind )
	{
		String str = indent( ind ) + "Node " + m_number + "\n";
        if( m_visiting )
            return str;
        m_visiting = true;
        if( m_erased )
        {
            str += indent( ind+2 ) + "erased\n";
//            return str;
        }
		str += indent( ind+2 ) + "Kind: " + m_kind + "\n";
        if( m_valueSet && m_value != null )
    		str += indent( ind+2 ) + "Value: " + m_value + "\n";
        if( m_op != null )
    		str += indent( ind+2 ) + "Op: " + m_op + "\n";
        if( m_x != null )
    		str += indent( ind+2 ) + "X: " + m_x + "\n";
        if( m_y != null )
    		str += indent( ind+2 ) + "Y: " + m_y + "\n";
        if( m_SSANumber > 0 )
            str += indent( ind+2 ) + "SSA: " + m_SSANumber + "\n";
        m_visiting = false;
		return str;
	}

	public String toString()
	{
		String str = "Node " + m_number;
		return str;
	}

    // print some spaces
	private String indent( int ind )
	{
		String str = "";
		for( int i=0; i<ind; i++ )
			str += " ";
		return str;
	}
}

