//package oberon_oo;

import java.lang.System;
import java_cup.runtime.*;
import sym;

class Utility {
  public static void assert
    (
     boolean expr
     )
      { 
	if (false == expr) {
	  throw (new Error("Error: Assertion failed." ));
	}
      }
  
  private static final String errorMsg[] = {
    "Error: Unmatched end-of-comment punctuation.",
    "Error: Unmatched start-of-comment punctuation.",
    "Error: Unclosed string.",
    "Error: Illegal character."
    };
  
  public static final int E_ENDCOMMENT = 0; 
  public static final int E_STARTCOMMENT = 1; 
  public static final int E_UNCLOSEDSTR = 2; 
  public static final int E_UNMATCHED = 3; 

  public static void error
    (
     int code
     )
      {
	System.out.println(errorMsg[code]);
      }
}

class Yytoken extends Symbol {
  Yytoken 
    (
     int index,
	 Object obj,
     int line,
     int charBegin,
     int charEnd
     )
      {
		super( index, charBegin, charEnd, obj );

		m_line = line;
      }

  public int m_line;
}

%%

%{
  private int comment_count = 0;
%} 

%eofval{
	return( new Yytoken( sym.EOF, "", yyline, yychar, yychar ));
%eofval}


%line
%char
%cup
%full

%class Scanner
%function next_token
%type Symbol

LETTER=[A-Za-z]
DIGIT=[0-9]
NONNEWLINE_WHITE_SPACE_CHAR=[\ \t\b\012]
WHITE_SPACE_CHAR=[\n\r\ \t\b\012]

INTEGER={DIGIT}*
IDENT={LETTER}({LETTER}|{DIGIT})*

%% 

"."			{ return( new Yytoken( sym.PERIOD,    new String(yytext()), yyline, yychar, yychar+1 )); }
","			{ return( new Yytoken( sym.COMMA,     new String(yytext()), yyline, yychar, yychar+1 )); }
";"			{ return( new Yytoken( sym.SEMICOLON, new String(yytext()), yyline, yychar, yychar+1 )); }
":"			{ return( new Yytoken( sym.COLON,     new String(yytext()), yyline, yychar, yychar+1 )); }
"("			{ return( new Yytoken( sym.LPAREN,    new String(yytext()), yyline, yychar, yychar+1 )); }
")"			{ return( new Yytoken( sym.RPAREN,    new String(yytext()), yyline, yychar, yychar+1 )); }
"["			{ return( new Yytoken( sym.LBRACKET,  new String(yytext()), yyline, yychar, yychar+1 )); }
"]"			{ return( new Yytoken( sym.RBRACKET,  new String(yytext()), yyline, yychar, yychar+1 )); }
"+"			{ return( new Yytoken( sym.PLUS,      new String(yytext()), yyline, yychar, yychar+1 )); }
"-"			{ return( new Yytoken( sym.MINUS,     new String(yytext()), yyline, yychar, yychar+1 )); }
"*"			{ return( new Yytoken( sym.TIMES,     new String(yytext()), yyline, yychar, yychar+1 )); }
"DIV"		{ return( new Yytoken( sym.DIVIDE,    new String(yytext()), yyline, yychar, yychar+3 )); }
"MOD"		{ return( new Yytoken( sym.MOD,       new String(yytext()), yyline, yychar, yychar+3 )); }
"="			{ return( new Yytoken( sym.EQ,        new String(yytext()), yyline, yychar, yychar+1 )); }
"#"			{ return( new Yytoken( sym.NE,        new String(yytext()), yyline, yychar, yychar+1 )); }
"<"			{ return( new Yytoken( sym.LT,        new String(yytext()), yyline, yychar, yychar+1 )); }
"<="		{ return( new Yytoken( sym.LE,        new String(yytext()), yyline, yychar, yychar+2 )); }
">"			{ return( new Yytoken( sym.GT,        new String(yytext()), yyline, yychar, yychar+1 )); }
">="		{ return( new Yytoken( sym.GE,        new String(yytext()), yyline, yychar, yychar+2 )); }
":="		{ return( new Yytoken( sym.ASSIGN,    new String(yytext()), yyline, yychar, yychar+2 )); }
"IF"		{ return( new Yytoken( sym.IF,        new String(yytext()), yyline, yychar, yychar+2 )); }
"THEN"		{ return( new Yytoken( sym.THEN,      new String(yytext()), yyline, yychar, yychar+4 )); }
"ELSIF"		{ return( new Yytoken( sym.ELSIF,     new String(yytext()), yyline, yychar, yychar+5 )); }
"ELSE"		{ return( new Yytoken( sym.ELSE,      new String(yytext()), yyline, yychar, yychar+4 )); }
"WHILE"		{ return( new Yytoken( sym.WHILE,     new String(yytext()), yyline, yychar, yychar+5 )); }
"DO"		{ return( new Yytoken( sym.DO,        new String(yytext()), yyline, yychar, yychar+2 )); }
"ARRAY"		{ return( new Yytoken( sym.ARRAY,     new String(yytext()), yyline, yychar, yychar+5 )); }
"OF"		{ return( new Yytoken( sym.OF,        new String(yytext()), yyline, yychar, yychar+2 )); }
"RECORD"	{ return( new Yytoken( sym.RECORD,    new String(yytext()), yyline, yychar, yychar+6 )); }
"CONST"		{ return( new Yytoken( sym.CONST,     new String(yytext()), yyline, yychar, yychar+5 )); }
"TYPE"		{ return( new Yytoken( sym.TYPE,      new String(yytext()), yyline, yychar, yychar+4 )); }
"VAR"		{ return( new Yytoken( sym.VAR,       new String(yytext()), yyline, yychar, yychar+3 )); }
"MODULE"	{ return( new Yytoken( sym.MODULE,    new String(yytext()), yyline, yychar, yychar+6 )); }
"BEGIN"		{ return( new Yytoken( sym.BEGIN,     new String(yytext()), yyline, yychar, yychar+5 )); }
"END"		{ return( new Yytoken( sym.END,       new String(yytext()), yyline, yychar, yychar+3 )); }

{WHITE_SPACE_CHAR}+ { }

{INTEGER} {
		return( new Yytoken( sym.NUMBER, new Integer(yytext()), yyline, yychar, yychar + yytext().length() ));
	}	

{IDENT} {
		return( new Yytoken( sym.IDENT, new String(yytext()), yyline, yychar, yychar + yytext().length() ));
	}	

